## $Id: level_report.py 10553 2013-08-29 05:24:10Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import grok
from zope.catalog.interfaces import ICatalog
from zope.component import queryUtility, getUtility
from zope.interface import implementer, Interface, Attribute
from waeup.kofa.interfaces import (
    IKofaUtils,
    academic_sessions_vocab, registration_states_vocab)
from waeup.kofa.interfaces import MessageFactory as _
from waeup.kofa.reports import IReport

class ILevelReport(IReport):

    session = Attribute('Session to report')
    level = Attribute('Level to report')
    faculty = Attribute('Faculty to report')
    department = Attribute('Department to report')
    creation_dt_string = Attribute('Human readable report creation datetime')

def get_students_by(faccode, depcode, session, level):
    """Get students in a certain department who registered courses
    in a certain session at a certain level.

    Returns a list of student data tuples.
    """
    site = grok.getSite()
    faccodes = tuple(sorted([x for x in site['faculties'].keys()],
                             key=lambda x: x.lower()))
    cat = queryUtility(ICatalog, name="students_catalog")
    result = cat.searchResults(
        depcode = (depcode, depcode), faccode = (faccode, faccode)
        )
    table = []
    for stud in result:
        if not stud['studycourse'].has_key(str(level)):
            continue
        level_obj = stud['studycourse'][str(level)]
        if level_obj.level_session != session:
            continue
        passed_params = level_obj.passed_params
        line = (stud.student_id,
                stud.matric_number,
                stud.display_fullname,
                level_obj.total_credits,
                passed_params[2],
                level_obj.gpa,
                passed_params[4],
                )
        table.append(line)
    return table
